#include "functions.h"

#include <QtCore/qmath.h>

namespace Functions {

bool isCounterClockwise(QPolygonF poly)
{
    qreal sum = 0;
    for(int i=0; i<poly.length(); i++) {
        QPointF p1 = poly[i];
        QPointF p2 = poly[(i+1) % poly.length()];
        sum += (p2.x() - p1.x()) * (p2.y() + p1.y());
    }
    return sum >= 0.0;
}

qreal triangleAngleCFromSidesABC(qreal a, qreal b, qreal c)
{
    return qAcos((a*a + b*b - c*c)/(2*a*b));
}

qreal vecLength(QPointF p)
{
    return qSqrt(p.x()*p.x() + p.y()*p.y());
}

qreal degAngleCounterClockwise(QPointF from, QPointF to)
{
    // note: this formula depends on the coordinate system choosen
    // this is the one for graphics (x to the right, y down)
    // for standart cartesian multiply with -1
    qreal tmp = qAtan2(from.y(), from.x()) - qAtan2(to.y(), to.x());
    if(tmp < 0) tmp += 2*M_PI;
    return qRadiansToDegrees(tmp);
}

qreal vecLengthSquared(QPointF p)
{
    return vecLength(p)*vecLength(p);
}

qreal distance(QPointF p1, QPointF p2)
{
    return vecLength(p1 - p2);
}

qreal dot(QPointF p1, QPointF p2)
{
    return p1.x()*p2.x() + p1.y()*p2.y();
}

qreal distToLine(QPointF p, QPointF l1, QPointF l2)
{
    QPointF l = l2 - l1;
    return qAbs(l.y()*p.x() - l.x()*p.y() + l2.x()*l1.y() - l2.y()*l1.x()) / distance(l1, l2);
}

qreal distToLineSegment(QPointF p, QPointF l1, QPointF l2)
{
    qreal lsq = vecLengthSquared(l2 - l1);
    if(lsq == 0.0) return distance(p, l1);
    qreal t = dot(p - l1, l2 - l1) / lsq;
    if(t < 0.0) return distance(p, l1);
    else if (t > 1.0) return distance(p, l2);
    QPointF proj = l1 + t*(l2 - l1);
    return distance(p, proj);
}

std::vector<cv::Point> getOpenCvPolygon(QPolygonF polygon)
{
    std::vector<cv::Point> _return;
    foreach (QPointF point, polygon) {
        _return.push_back(cv::Point(round(point.x()),round(point.y())));
    }
    return _return;
}

QPolygonF getQPolygonF(std::vector<cv::Point> polygon)
{
    QPolygonF _return;
    for (std::vector<cv::Point>::iterator it = polygon.begin() ; it != polygon.end(); ++it){
            _return << QPointF(it->x, it->y);
    }
    return _return;
}

QPolygonF transformPolygonAffine(QPolygonF polygon, Eigen::Affine2d affineTransform)
{
    QPolygonF _return;
    foreach(QPointF p, polygon){
        Eigen::Vector2d transformedPoint = affineTransform * Eigen::Vector2d(p.x(), p.y());
        _return << QPointF(transformedPoint(0), transformedPoint(1));
    }
    return _return;
}

}
